import { Column, Entity, JoinColumn, ManyToOne, OneToMany, PrimaryGeneratedColumn } from 'typeorm';
import { Lift } from './Lift';
import { Site } from './Site';

@Entity()
export class Building {
  constructor(config?: { [P in keyof Building]?: Building[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn()
  id: number;

  @Column()
  name: string;

  @Column('int')
  lift_qty: number;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  // Relation
  @ManyToOne(() => Site, site => site.buildings)
  @JoinColumn({ name: 'site_id' })
  site: Site;

  @OneToMany(() => Lift, lift => lift.building)
  lifts: Lift[];
}
