import { Column, Entity, PrimaryGeneratedColumn, OneToMany } from 'typeorm';
import { EStatus } from '../utils/enumConstant';
import { Site } from './Site';
import { Profile } from './Profile';
import { ProfileClient } from './ProfileClient';
import { UserRegion } from './UserRegion';

@Entity()
export class IsoCode {
  constructor(config?: { [P in keyof IsoCode]?: IsoCode[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn()
  id: number;

  @Column({ length: 50 })
  code: string;

  @Column()
  name: string;

  @Column('tinyint', { default: EStatus.active })
  status: EStatus;

  @Column('decimal', { precision: 11, scale: 8, nullable: true })
  latitude: number;

  @Column('decimal', { precision: 11, scale: 8, nullable: true })
  longitude: number;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  @OneToMany(() => Site, site => site.region)
  sites: Site[];

  @OneToMany(() => UserRegion, user_region => user_region.region)
  user_regions: UserRegion[];
}
