import { Column, Entity, JoinColumn, ManyToOne, OneToMany, PrimaryGeneratedColumn } from 'typeorm';
import { EStatus } from '../utils/enumConstant';
import { Building } from './Building';
import { LiftAssignee } from './LiftAssignee';
import { Site } from './Site';
import { Task } from './Task';

@Entity()
export class Lift {
  constructor(config?: { [P in keyof Lift]?: Lift[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn()
  id: number;

  @ManyToOne(() => Site, site => site.lifts)
  @JoinColumn({ name: 'site_id' })
  site: Site;

  @ManyToOne(() => Building, buiding => buiding.lifts)
  @JoinColumn({ name: 'building_id' })
  building: Building;

  @Column('int')
  lift_no: number;

  @Column({ unique: true })
  lift_contract_no: string;

  @Column({ type: 'int', nullable: true })
  line_no: number;

  @Column()
  model_no: string;

  @Column({ type: 'int', nullable: true })
  machine_room_no: number;

  @Column()
  floor_list: string;

  @Column({
    nullable: true,
    type: 'varchar',
    length: 5000,
    transformer: {
      to(value: any): any {
        if (['[object Object]', '[object Array]'].includes(Object.prototype.toString.call(value))) {
          return JSON.stringify(value);
        } else {
          return value;
        }
      },
      from(value: any): any {
        try {
          return JSON.parse(value);
        } catch (e) {
          if (value) {
            return [value];
          }
          return [];
        }
      },
    },
  })
  description: any;

  @Column({ type: 'tinyint', nullable: false })
  type: number;

  @Column({
    type: 'tinyint',
    nullable: false,
    default: 0,
    transformer: {
      to(value: any): any {
        if (Object.prototype.toString.call(value) === '[object Boolean]') {
          return value ? 1 : 0;
        }

        return value ? 1 : 0;
      },
      from(value: any): any {
        return !!value;
      },
    },
  })
  scope_lift_monitoring: number | boolean;

  @Column({ type: 'timestamp', nullable: false })
  warranty_from: Date;

  @Column({ type: 'timestamp', nullable: false })
  warranty_to: Date;

  @Column('tinyint')
  status: number;

  @Column('tinyint', { default: 1 })
  active: EStatus;

  @Column('decimal', { precision: 11, scale: 8 })
  latitude: number;

  @Column('decimal', { precision: 11, scale: 8 })
  longitude: number;

  @Column('decimal', { precision: 5, scale: 2, nullable: true })
  performance: number;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  last_checked_at: Date;

  // Relation
  @OneToMany(() => LiftAssignee, lift_assignee => lift_assignee.lift)
  assignees: LiftAssignee[];

  @OneToMany(() => Task, task => task.lift)
  tasks: Task[];
}
