import { Column, Entity, PrimaryGeneratedColumn } from 'typeorm';
import { EStatus } from '../utils/enumConstant';

@Entity()
export class Media {
  constructor(config?: { [P in keyof Media]?: Media[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn('uuid')
  id: string;

  @Column()
  name: string;

  @Column({ length: 50 })
  type: string;

  @Column()
  path: string;

  @Column('varchar', { nullable: true })
  external_url: string;

  @Column('text', {
    nullable: true,
    transformer: {
      from(val: string | undefined | null) {
        try {
          return JSON.parse(val);
        } catch (e) {
          return null;
        }
      },
      to(val: string | undefined | null) {
        return JSON.stringify(val);
      },
    },
  })
  metadata: string;

  @Column('tinyint')
  status: EStatus;

  @Column('simple-array')
  available_resolution: string[];

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;
}
