import { Column, Entity, OneToOne, JoinColumn, ManyToOne } from 'typeorm';
import { EGender } from '../utils/enumConstant';
import { IsoCode } from './IsoCode';
import { User } from './User';

@Entity()
export class Profile {
  constructor(config?: { [P in keyof Profile]?: Profile[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @OneToOne(() => User, { primary: true })
  @JoinColumn({ name: 'user_id' })
  user: User;

  @Column({ nullable: true })
  full_name: string;

  @Column({ nullable: true, length: 1000 })
  address: string;

  @Column('varchar', { length: 10, nullable: true })
  gender: EGender;

  @Column({ nullable: true })
  dob: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;
}
