import { Column, Entity, JoinColumn, ManyToOne, OneToMany, PrimaryGeneratedColumn } from 'typeorm';
import { GroupRole } from './GroupRole';
import { PermissionRole } from './PermissionRole';
import { UserRole } from './UserRole';

@Entity()
export class Role {
  constructor(config?: { [P in keyof Role]?: Role[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn()
  id: number;

  @ManyToOne(() => GroupRole, group_role => group_role.roles)
  @JoinColumn({ name: 'group_role_id' })
  group: GroupRole;

  @Column({ nullable: false, unique: true })
  slug: string;

  @Column({ nullable: false })
  name: string;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  @OneToMany(() => PermissionRole, permissionRoles => permissionRoles.role)
  permission_roles: PermissionRole[];

  @OneToMany(() => UserRole, userRoles => userRoles.role)
  user_roles: UserRole[];
}
