import { Column, Entity, Index, JoinColumn, ManyToOne, OneToMany, PrimaryGeneratedColumn } from 'typeorm';
import { Lift } from './Lift';
import { IsoCode } from './IsoCode';
import { UserInCharge } from './UserInCharge';
import { Building } from './Building';
import { EStatus } from '../utils/enumConstant';

@Entity()
export class Site {
  constructor(config?: { [P in keyof Site]?: Site[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn()
  id: number;

  @Index({ unique: true })
  @Column()
  contract_no: string;

  @ManyToOne(() => IsoCode, iso_code => iso_code.sites)
  @JoinColumn({ name: 'iso_code_id' })
  region: IsoCode;

  @Column()
  site_code: string;

  @Column({ nullable: true, length: 1000 })
  site_name: string;

  @Column({ nullable: true, length: 1000 })
  site_address: string;

  @Column({ nullable: true })
  thumbnail: string;

  @Column('int')
  dong_qty: number;

  @Column('int')
  car_qty: number;

  @Column('decimal', { precision: 11, scale: 8 })
  latitude: number;

  @Column('decimal', { precision: 11, scale: 8 })
  longitude: number;

  @Column({ nullable: true, length: 5000 })
  description: string;

  @Column({ type: 'timestamp', nullable: false })
  installed_date: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  // Relation
  @OneToMany(() => Building, building => building.site)
  buildings: Building[];

  @OneToMany(() => Lift, lift => lift.site)
  lifts: Lift[];

  @OneToMany(() => UserInCharge, userInCharge => userInCharge.site)
  users_in_charge: UserInCharge[];

  @Column('tinyint', { default: 1 })
  active: EStatus;
}
