import { Column, Entity, Index, JoinColumn, ManyToOne, OneToMany, PrimaryGeneratedColumn } from 'typeorm';
import { Lift } from './Lift';
import { TaskMedia } from './TaskMedia';
import { TaskTime } from './TaskTime';
import { User } from './User';
import { Category } from './Category';

@Entity()
export class Task {
  constructor(config?: { [P in keyof Task]?: Task[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn()
  id: number;

  @Column({ nullable: true, length: 5000 })
  issue: string;

  @Column({
    nullable: true,
    length: 5000,
    type: 'varchar',
    transformer: {
      to(value: any): any {
        if (Object.prototype.toString.call(value) === '[object Array]') {
          return JSON.stringify(value);
        } else {
          return value;
        }
      },
      from(value: any): any {
        try {
          return JSON.parse(value);
        } catch (e) {
          return value ? [value] : [];
        }
      },
    },
  })
  error_suggestion: any;

  @Column({ type: 'varchar', length: 5000, nullable: true })
  engineer_report: string;

  @Column('tinyint')
  status: number;

  @Column('tinyint')
  type: number;

  @Column('tinyint', { nullable: true })
  rate: number;

  @Column({ nullable: true, length: 1000 })
  feedback: string;

  @Column({ nullable: true })
  @Index()
  alarm_id: string;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  // Relation
  @ManyToOne(() => Lift, lift => lift.tasks)
  @JoinColumn({ name: 'lift_id' })
  lift: Lift;

  @ManyToOne(() => User, user => user.client_tasks)
  @JoinColumn({ name: 'client_id' })
  client: User;

  @ManyToOne(() => Category, cate => cate.tasks)
  @JoinColumn({ name: 'category_id' })
  category: Category;

  @ManyToOne(() => User, user => user.engineer_task)
  @JoinColumn({ name: 'engineer_id' })
  engineer: User;

  @ManyToOne(() => User, user => user.operator_task)
  @JoinColumn({ name: 'operator_id' })
  operator: User;

  @OneToMany(() => TaskTime, task_time => task_time.task)
  times: TaskTime[];

  @OneToMany(() => TaskMedia, task_media => task_media.task)
  media_list: TaskMedia[];
}
