import { Column, Entity, OneToMany, OneToOne, PrimaryGeneratedColumn } from 'typeorm';
import { EUserStatus } from '../utils/enumConstant';
import { LiftAssignee } from './LiftAssignee';
import { Profile } from './Profile';
import { ProfileClient } from './ProfileClient';
import { SocialNetworkInfo } from './SocialNetworkInfo';
import { Task } from './Task';
import { UserMeta } from './UserMeta';
import { UserRole } from './UserRole';
import { UserInCharge } from './UserInCharge';
import { UserRegion } from './UserRegion';
@Entity()
export class User {
  constructor(config?: { [P in keyof User]?: User[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @PrimaryGeneratedColumn('uuid')
  id: string;

  @Column({ unique: true })
  email: string;

  @Column({ unique: true })
  username: string;

  @Column({ unique: true })
  phone: string;

  @Column({ default: '+84' })
  phone_dial_code: string;

  @Column({ unique: true })
  slug: string;

  @Column({ select: false })
  password: string;

  @Column({ nullable: true })
  fcm_token: string;

  @Column({ nullable: true })
  locale: string;

  @Column('decimal', { precision: 11, scale: 8, nullable: true })
  latitude: number;

  @Column('decimal', { precision: 11, scale: 8, nullable: true })
  longitude: number;

  @Column({ type: 'timestamp', nullable: true })
  last_location_at: Date;

  @Column({ default: '+07' })
  timezone: string;

  @Column('tinyint', { default: 1 })
  status: EUserStatus;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;

  // Relation
  @OneToOne(() => Profile, profile => profile.user)
  profile: Profile;

  @OneToOne(() => ProfileClient, profile_client => profile_client.user)
  profile_client: ProfileClient;

  @OneToMany(() => SocialNetworkInfo, social_network_info => social_network_info.user)
  social_network_info: SocialNetworkInfo[];

  @OneToMany(() => UserMeta, user_meta => user_meta.user)
  metadata: UserMeta[];

  @OneToMany(() => UserRole, user_role => user_role.user)
  user_roles: UserRole[];

  @OneToMany(() => LiftAssignee, lift_assignee => lift_assignee.user)
  assignees: LiftAssignee[];

  @OneToMany(() => Task, task => task.engineer)
  engineer_task: Task[];

  @OneToMany(() => Task, task => task.operator)
  operator_task: Task[];

  @OneToMany(() => Task, task => task.client)
  client_tasks: Task[];

  @OneToMany(() => UserInCharge, userInCharge => userInCharge.user)
  users_in_charge: UserInCharge[];

  @OneToMany(() => UserRegion, user_region => user_region.user)
  user_regions: UserRegion[];
}
