import { Column, Entity, JoinColumn, ManyToOne } from 'typeorm';
import { User } from './User';

@Entity()
export class UserMeta {
  constructor(config?: { [P in keyof UserMeta]?: UserMeta[P] }) {
    if (config) {
      Object.assign(this, { ...config });
    }
  }

  @ManyToOne(() => User, user => user.metadata, {
    primary: true,
    orphanedRowAction: 'delete',
  })
  @JoinColumn({ name: 'user_id' })
  user: User;

  @Column({ nullable: false, primary: true })
  meta_key: string;

  @Column({
    type: 'varchar',
    transformer: {
      to(value: any): any {
        if (['[object Object]', '[object Array]'].includes(Object.prototype.toString.call(value))) {
          return JSON.stringify(value);
        } else {
          return value;
        }
      },
      from(value: any): any {
        try {
          return JSON.parse(value);
        } catch (e) {
          return value;
        }
      },
    },
  })
  meta_value: any;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  created_at: Date;

  @Column({ type: 'timestamp', nullable: false, default: () => 'CURRENT_TIMESTAMP' })
  updated_at: Date;
}
